import { ChainCache } from 'cache/ChainCache'
import { GAS_BOOST } from 'common/Constants'
const abi = require('abis/NftDistributor.json').abi

export class DistributorReactor {
  /**
   * 发布NFT列表
   */
  async publishAirdropList({
    chain,
    address,
    to,
    nftList,
    encodeABI = false,
  }: {
    chain: number
    address: string
    to: string
    nftList: string[]
    encodeABI?: boolean
  }) {
    const { instance, account } = new ChainCache().getInstances(chain, address, abi)
    if (encodeABI) {
      return instance.methods.addNFTData(to, nftList).encodeABI()
    }
    let gas = await instance.methods.addNFTData(to, nftList).estimateGas({ from: account.address })
    let res = await instance.methods.addNFTData(to, nftList).send({ gas: (gas * GAS_BOOST) | 0 })
    return res
  }
  /**
   * mint nft to user
   */
  async mintNft({
    chain,
    address,
    to,
    count,
    encodeABI = false,
  }: {
    chain: number
    address?: string
    to: string
    count: number
    encodeABI?: boolean
  }) {
    const { instance, account } = new ChainCache().getInstances(chain, address, abi)
    const countNft = count + ''
    if (encodeABI) {
      return instance.methods.mintToUser(to, countNft).encodeABI()
    }
    let gas = await instance.methods.mintToUser(to, countNft).estimateGas({ from: account.address })
    let res = await instance.methods.mintToUser(to, countNft).send({ gas: (gas * GAS_BOOST) | 0 })
    return res
  }
  /**
   * 查询用户可mint的数量
   */
  async getMintableCount({ chain, contract, user }: { chain: number; contract?: string; user: string }) {
    const { instance } = new ChainCache().getInstances(chain, contract, abi)
    return await instance.methods.getMintableCount(user).call()
  }
}
