import { ChainCache } from 'cache/ChainCache'
import { GAS_BOOST } from 'common/Constants'
import Web3 from 'web3'

const abi = require('abis/ERC20.json').abi
export class ERC20Reactor {
  async transfer({
    chain,
    address,
    from,
    to,
    amount,
    encodeABI = false,
  }: {
    chain: number
    address: string
    from: string
    to: string
    amount: number | string
    encodeABI: boolean
  }) {
    const { instance, account } = new ChainCache().getInstances(chain, address, abi)
    const amountBN = Web3.utils.toBN(Web3.utils.toWei(amount + ''))
    if (encodeABI) {
      return instance.methods.transfer(to, amountBN).encodeABI()
    }
    let gas = await instance.methods.transferFrom(from, to, amountBN).estimateGas({ from: account.address })
    return instance.methods.transferFrom(from, to, amountBN).send({
      gas: (gas * GAS_BOOST) | 0,
    })
  }

  async mint({
    chain,
    address,
    to,
    amount,
    encodeABI = false,
  }: {
    chain: number
    address?: string
    to: string
    amount: string
    encodeABI: boolean
  }) {
    const { instance, account } = new ChainCache().getInstances(chain, address, abi)
    let amountBN = Web3.utils.toBN(Web3.utils.toWei(amount + ''))
    if (encodeABI) {
      return instance.methods.mint(to, amountBN).encodeABI()
    }
    let gas = await instance.methods.mint(to, amountBN).estimateGas({ from: account.address })
    return instance.methods.mint(to, amountBN).send({ gas: (gas * GAS_BOOST) | 0 })
  }
}
