import { ChainCache } from 'cache/ChainCache'
import { GAS_BOOST } from 'common/Constants'

const abiBadge = require('abis/BEBadge.json').abi
const abiNFT = require('abis/NFT.json').abi

export class ERC721Reactor {
  async transfer({
    chain,
    address,
    from,
    to,
    tokenId,
    encodeABI = false,
  }: {
    chain: number
    address?: string
    from: string
    to: string
    tokenId: string
    encodeABI?: boolean
  }) {
    const { instance, account } = new ChainCache().getInstances(chain, address, abiNFT)
    if (encodeABI) {
      return instance.methods.safeTransferFrom(from, to, tokenId).encodeABI()
    }
    let gas = await instance.methods.safeTransferFrom(from, to, tokenId).estimateGas({ from: account.address })
    return instance.methods.safeTransferFrom(from, to, tokenId).send({
      gas: (gas * GAS_BOOST) | 0,
    })
  }

  async mintNFT({
    chain,
    address,
    to,
    tokenIds,
    encodeABI = false,
  }: {
    chain: number
    address?: string
    to: string
    tokenIds: string[]
    encodeABI?: boolean
  }) {
    const { instance, account } = new ChainCache().getInstances(chain, address, abiNFT)

    if (encodeABI) {
      return instance.methods.batchMint(to, tokenIds).encodeABI()
    }
    let gas = await instance.methods.batchMint(to, tokenIds).estimateGas({ from: account.address })
    return instance.methods.batchMint(to, tokenIds).send({ gas: (gas * GAS_BOOST) | 0 })
  }

  async mintBadge({
    chain,
    address,
    to,
    amount,
    encodeABI = false,
  }: {
    chain: number
    address?: string
    to: string
    amount: number
    encodeABI?: boolean
  }) {
    const { instance, account } = new ChainCache().getInstances(chain, address, abiBadge)
    const countStr = amount + ''
    if (encodeABI) {
      return instance.methods.batchMint(to, countStr).encodeABI()
    }
    let gas = await instance.methods.batchMint(to, countStr).estimateGas({ from: account.address })
    return instance.methods.batchMint(to, countStr).send({ gas: (gas * GAS_BOOST) | 0 })
  }
}
