import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'

@dbconn()
@index({ address: 1 }, { unique: false })
@index({ transactionHash: 1, from: 1, to: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'ft_transfer_event', timestamps: true },
})
export class FtTransferEventClass extends BaseModule {
  @prop({ required: true })
  public address!: string
  @prop()
  public event: string
  @prop({ required: true })
  public transactionHash: string
  @prop()
  public blockNumber: number
  @prop()
  public blockHash: string
  @prop()
  public removed: boolean
  @prop()
  public from: string
  @prop()
  public to: string
  @prop()
  public amount: string
  @prop()
  public blockTime: number

  public static async saveEvent(event: any) {
    const amount = event.returnValues?.value
    if (amount == undefined) {
      return
    }
    const from = event.returnValues?.from
    const to = event.returnValues?.to
    const data = {
      address: event.address,
      blockNumber: event.blockNumber,
      blockHash: event.blockHash,
      removed: event.removed,
      event: event.event,
      from,
      to,
      transactionHash: event.transactionHash,
      amount,
      blockTime: event.timestamp * 1000
    }
    return FtTransferEvent.insertOrUpdate({ transactionHash: event.transactionHash, amount, from, to }, data)
  }
}

export const FtTransferEvent = getModelForClass(FtTransferEventClass, {
  existingConnection: FtTransferEventClass['db'],
})
