import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'

@dbconn()
@index({ tokenId: 1 }, { unique: false })
@index({ transactionHash: 1, tokenId: 1, from: 1, to: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'nft_transfer_event', timestamps: true },
})
export class NftTransferEventClass extends BaseModule {
  @prop({ required: true })
  public address!: string
  @prop()
  public event: string
  @prop({ required: true })
  public transactionHash: string
  @prop()
  public blockNumber: number
  @prop()
  public blockHash: string
  @prop()
  public removed: boolean
  @prop()
  public from: string
  @prop()
  public to: string
  @prop()
  public tokenId: string
  @prop()
  public blockTime: number
  @prop({ default: 0 })
  public version: number

  public static async saveEvent(event: any) {
    if (!event.success) {
      return
    }
    const tokenId = event.tokenId
    if (!tokenId) {
      return
    }
    const from = event.source
    const to = event.target
    const data = {
      address: event.tokenAddress,
      blockNumber: event.blockHeight,
      removed: event.removed,
      from,
      to,
      transactionHash: event.hash,
      tokenId,
      blockTime: new Date(event.time).getTime(),
      $inc: { version: 1 },
    }

    return NftTransferEvent.insertOrUpdate({ transactionHash: event.hash, tokenId, from, to }, data)
  }
}

export const NftTransferEvent = getModelForClass(NftTransferEventClass, {
  existingConnection: NftTransferEventClass['db'],
})
