import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'

@dbconn()
@index({ chain: 1, transactionHash: 1, scheduleId: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'schedule_added_event', timestamps: true },
})
export class ScheduledAddedEventClass extends BaseModule {
  @prop({ required: true })
  public address!: string
  @prop()
  public chain: number
  @prop()
  public event: string
  @prop({ required: true })
  public transactionHash: string
  @prop()
  public blockNumber: number
  @prop()
  public blockHash: string
  @prop()
  public removed: boolean
  @prop()
  public operater: string
  @prop()
  public scheduleId: string
  @prop()
  public blockTime: number
  @prop({ default: 0 })
  public version: number

  public static async saveEvent(event: any) {
    if (event.removed) {
      return
    }

    const data = {
      address: event.address,
      blockNumber: event.blockNumber,
      removed: event.removed,
      operater: event.returnValues.sender,
      transactionHash: event.transactionHash,
      blockTime: new Date(event.timestamp).getTime(),
      $inc: { version: 1 },
    }

    return ScheduledAddedEvent.insertOrUpdate(
      { transactionHash: event.transactionHash, scheduleId: event.returnValues.id, chain: event.chain },
      data,
    )
  }
}

export const ScheduledAddedEvent = getModelForClass(ScheduledAddedEventClass, {
  existingConnection: ScheduledAddedEventClass['db'],
})
