import { AsyncQueue, createAsyncQueue } from 'common/AsyncQueue'
import { singleton } from 'decorators/singleton'
import { DocumentType } from '@typegoose/typegoose'
import { ReqTaskStatus, RequestTaskClass } from 'models/RequestTask'
import Web3 from 'web3'
import { isSuccessfulTransaction, waitTransaction } from 'chain/TransactionConfirm'
import { ChainTask } from 'models/ChainTask'
import { ChainQueue } from './chain.queue'
import logger from 'logger/logger'
import { ExecQueue } from './exec.queue'

@singleton
export class ConfirmQueue {
  private queue: AsyncQueue
  private web3: Web3

  constructor(web3: Web3) {
    this.queue = createAsyncQueue()
    this.web3 = web3
  }

  public addTaskToQueue(task: DocumentType<RequestTaskClass>) {
    this.queue.push(async () => {
      try {
        let receipt = await waitTransaction(this.web3, task.txHash)
        logger.info(`receipt confirmed: status: ${receipt.statue}, txhash: ${task.txHash}`)
        if (isSuccessfulTransaction(receipt)) {
          if (task.status === ReqTaskStatus.WAIT_CONFIRM) {
            task.status = ReqTaskStatus.WAIT_EXEC
            task.endScheduleTime = Date.now()
            await task.save()
            await ChainTask.checkScheduleStatus(task.chainTaskId)
          } else if (task.status === ReqTaskStatus.WAIT_EXEC_CONFIRM) {
            task.status = ReqTaskStatus.SUCCESS
            task.endExecTime = Date.now()
            await task.save()
            await ChainTask.checkStatus(task.chainTaskId)
          }
        } else {
          if (task.status === ReqTaskStatus.WAIT_CONFIRM) {
            task.status = ReqTaskStatus.SCHEDULE_REVERT
            new ChainQueue().addTaskToQueue(task)
          } else if (task.status === ReqTaskStatus.WAIT_EXEC_CONFIRM) {
            task.status = ReqTaskStatus.EXEC_REVERT
            new ExecQueue().addTaskToQueue(task)
          }
          await task.save()
        }
      } catch (err) {
        logger.error(err)
        task.errMsg.push(err)
        await task.save()
        if (task.status === ReqTaskStatus.WAIT_CONFIRM) {
          new ChainQueue().addTaskToQueue(task)
        } else if (task.status === ReqTaskStatus.WAIT_EXEC_CONFIRM) {
          new ExecQueue().addTaskToQueue(task)
        }
      }
    })
  }
}
