import { AsyncQueue, createAsyncQueue } from 'common/AsyncQueue'
import { singleton } from 'decorators/singleton'
import { BlockChain } from 'chain/BlockChain'
import logger from 'logger/logger'
import { ReqTaskStatus, RequestTaskClass } from 'models/RequestTask'
import { DocumentType } from '@typegoose/typegoose'
import { ChainTask } from 'models/ChainTask'
import { ChainCache } from 'cache/ChainCache'

const EXCLUDE_STATUS = [
  0, //ReqTaskStatus.NOTSTART,
  5, //ReqTaskStatus.SUCCESS,
  1, //ReqTaskStatus.PEDING,
  2, //ReqTaskStatus.WAIT_CONFIRM,
  8, //ReqTaskStatus.SCHEDULE_REVERT,
]
@singleton
export class ExecQueue {
  private queue: AsyncQueue
  private blockChain: BlockChain

  constructor() {
    this.queue = createAsyncQueue()
    this.blockChain = new BlockChain()
  }

  public async addTaskToQueue(subTask: DocumentType<RequestTaskClass>, force = false) {
    if (!force && EXCLUDE_STATUS.indexOf(subTask.status) >= 0) {
      return
    }
    if (!force && subTask.maxTryCount && subTask.execCount > subTask.maxTryCount) {
      subTask.status = ReqTaskStatus.ERROR
      await subTask.save()
      await ChainTask.checkStatus(subTask.chainTaskId)
      return
    }
    if (!force && subTask.status === ReqTaskStatus.WAIT_EXEC_CONFIRM) {
      new ChainCache().getConfirmQueue(subTask.chain).addTaskToQueue(subTask)
      return
    }
    this.queue.push(async () => {
      subTask.execCount += 1
      try {
        try {
          await subTask.execSchdule()
        } catch (reqerr) {
          logger.info(reqerr)
          subTask.errMsg.push(JSON.stringify(reqerr))
          await subTask.save()
          // TODO:: 要排除数据已经提交到链上, 返回过程中的网络错误
          this.addTaskToQueue(subTask)
          return
        }
        new ChainCache().getConfirmQueue(subTask.chain).addTaskToQueue(subTask)
      } catch (err) {
        logger.error('error add task: ' + err)
        subTask.errMsg.push(err)
        await subTask.save()
      }
    })
  }
}
