import { AsyncQueue, createAsyncQueue } from 'common/AsyncQueue'
import { singleton } from 'decorators/singleton'
import { DocumentType } from '@typegoose/typegoose'
import logger from 'logger/logger'
import { ChainTaskClass } from 'models/ChainTask'
import { MailService } from 'service/mail.service'
import { Deferred } from 'utils/promise.util'
import { CONFIRM_MAIL_HTML } from 'common/Constants'
import { ChainCache } from 'cache/ChainCache'
import assert from 'assert'

export interface IMailData {
  from?: string
  to?: string
  subject?: string
  text?: string
  html?: string
}

const DEFAULT_MSG_DATA: IMailData = {
  from: 'CEBG <noreply@cebg.games>',
  subject: 'CEBG',
}

@singleton
export class MailQueue {
  private queue: AsyncQueue
  private chain: number
  constructor(chain: number) {
    this.chain = chain
    this.queue = createAsyncQueue()
  }

  public async addTaskToQueue(task: DocumentType<ChainTaskClass>) {
    let html = CONFIRM_MAIL_HTML
    html = html.replace('{{title}}', task.name)
    html = html.replace('{{desc}}', task.desc)
    let link = `${process.env.WEB_BASE_URL}/pages/redirect.html?id=${task.id}`
    html = html.replace('{{link}}', link)
    let link2 = `${process.env.WEB_BASE_URL}/pages/confirm.html?id=${task.id}`
    html = html.replace('{{link2}}', link2)
    let subject = `${task.starterName} 申请 ${task.name}`
    let cfg = new ChainCache().chainArray.find(c => c.id === this.chain)
    assert(cfg, `chain ${this.chain} not found`)
    let data: any = { html, subject }
    Object(DEFAULT_MSG_DATA).zssign(data)
    let deferred = new Deferred()
    this.queue.push(async () => {
      try {
        let mails = cfg.mail.split(',')
        let infos: any[] = []
        for (let mail of mails) {
          data.to = mail
          let info = await new MailService().send(data)
          infos.push(info)
          logger.info(
            `send mail success:: from: ${data.from}, to: ${data.to}, subject: ${data.subject}, messageId: ${info.messageId}`,
          )
        }
        deferred.resolve(infos)
      } catch (err) {
        logger.info(`send mail error:: from: ${data.from}, to: ${data.to}, subject: ${data.subject}`)
        logger.error(err)
        deferred.reject(err)
      }
    })
    return deferred.promise
  }
}
