import { singleton } from 'decorators/singleton'
import logger from 'logger/logger'
import { ChainTask } from 'models/ChainTask'
import * as schedule from 'node-schedule'
import { TaskQueue } from 'queue/task.queue'
import { WechatWorkService } from 'service/wechatwork.service'

@singleton
export default class WxTaskSchedule {
  async parseAllRecord() {
    let detail: any = await new WechatWorkService().queryTasks()
    if (detail.errcode) {
      logger.info('approval list error, code: ' + detail.errcode + ' errmsg: ' + detail.errmsg)
      return
    }
    const { sp_no_list } = detail
    for (let spNo of sp_no_list) {
      let record = await ChainTask.findOne({ taskId: spNo })
      if (record) {
        continue
      }
      logger.info('got one task: ' + spNo)
      new TaskQueue().addTaskToQueue(spNo)
    }
  }
  scheduleAll() {
    const job = schedule.scheduleJob(' */30 * * * * *', async () => {
      await this.parseAllRecord()
    })
    this.parseAllRecord()
  }
}
